#include "sn_PushableBlock.h"

void sn_PushableBlock::init(sn_3DmodelCollection * p3DModelCollection, sn_MovementArray * pMA,
							string modelFilename, string movementMapFilename)
{

	
	pModels = p3DModelCollection;
	pModels->load(modelFilename);
	modelID = pModels->getModelIndex(modelFilename);
	pMovementArray = pMA;
	originalMovementDeltas.init(pMovementArray);
	blockMovementDeltas.init(pMovementArray);
	blockMovementDeltas.load(movementMapFilename);
	
	x = y = z = 1;
	targetX = targetY = targetZ = 1;
	oldX = oldY = oldZ = 1;
	deltaX = deltaY = deltaZ = 0;
	
	//recordOriginalMovementDeltas();
	
	moveSpeed = 0.009;
	layer = 2;
	facing = 0;
	height = 1.0f;
	halfheight = height/2.0f;
	collider.init(0,0,halfheight,1.9,1.9,0.9);
	
	fallingOffStage  = false;
	gone = false;
	arrived = true;
}

void sn_PushableBlock::draw( float rotation )
{
	if (!gone)
		pModels->draw(modelID, facing);
}

void sn_PushableBlock::drawAlpha( float rotation )
{
	if (!gone)
		pModels->drawAlpha(modelID, facing);
}

void sn_PushableBlock::moveTo( float newX, float newY, float newZ )
{
		x = newX; y = newY; z = newZ;
		oldX = x; oldY = y; oldZ = z;
		collider.moveTo( newX, newY, newZ+halfheight );
		//arrived = true;
}

void sn_PushableBlock::moveBy( float xOffset, float yOffset, float zOffset)
{
		targetX = x+xOffset; targetY = y+yOffset; targetZ = z+zOffset;
		if (xOffset > 0) deltaX = moveSpeed; if (xOffset < 0) deltaX = -moveSpeed;
		if (yOffset > 0) deltaY = moveSpeed; if (yOffset < 0) deltaY = -moveSpeed;
		if (zOffset > 0) deltaZ = moveSpeed; if (zOffset < 0) deltaZ = -moveSpeed;
		moving = true;
}

void sn_PushableBlock::receivePush( int direction )
{
	if (direction == 0)
	{
		moveBy( 0,1,0);
	}
	else if (direction == 1)
	{
		moveBy (1,0,0);
	}
	else if (direction == 2)
	{
		moveBy (0,-1,0);
	}
	else if (direction == 3)
	{
		moveBy (-1,0,0);
	}
}

sn_Collider sn_PushableBlock::getCollider()
{
	return collider;
}

bool sn_PushableBlock::isActive()
{
	return moving;
}

void sn_PushableBlock::fallOffStage()
{
	fallingOffStage = true;
}

bool sn_PushableBlock::isFallingOffStage()
{
	return fallingOffStage;
}

void sn_PushableBlock::setGone( bool tf )
{
	gone = tf;
	if (gone)
	{
		moving = false;
		arrived = false;
	}
	
}

bool sn_PushableBlock::isGone()
{
	return gone;
}

void sn_PushableBlock::setFalling( bool tf )
{
	falling = tf;
}

bool sn_PushableBlock::isFalling()
{
	return falling;
}

//Checking this makes it false afterwords
bool sn_PushableBlock::hasArrived()
{
	if (arrived)
	{
		arrived = false;
		return true;
	}
	else
	{
		return false;
	}
}

void sn_PushableBlock::animate( unsigned int ticksPassed )
{
	if (moving && !gone)
	{
		x += deltaX * ticksPassed;
		if (deltaX > 0 && x > targetX) {x = targetX; deltaX = 0;}
		if (deltaX < 0 && x < targetX) {x = targetX; deltaX = 0;}
		
		y += deltaY * ticksPassed;
		if (deltaY > 0 && y > targetY) {y = targetY; deltaY = 0;}
		if (deltaY < 0 && y < targetY) {y = targetY; deltaY = 0;}
		
		//if (falling) {deltaZ -= 0.0004;}
		z += deltaZ * ticksPassed;
		if (deltaZ > 0 && z > targetZ) {z = targetZ; deltaZ = 0;}
		if (deltaZ < 0 && z < targetZ) {z = targetZ; deltaZ = 0;}
		
		collider.moveTo(x,y,z+halfheight);
		
		if (deltaX == 0 && deltaY == 0 && deltaZ == 0)
		{
			moving = false;
			arrived = true;
		}
	}
}



void sn_PushableBlock::updateMovementDeltas()
{
	if (gone || fallingOffStage ) return;
	//restore originals that were replaced when block had moved here
	originalMovementDeltas.applyAt((int)oldX, (int)oldY, (int)oldZ, facing);
	oldX = x; oldY = y; oldZ = z; //update old vertex so it can be restored later
	//record the original deltas that are under where the block is being moved to
	recordOriginalMovementDeltas();
	//finally, apply the block into the movement array writing over the previous values
	blockMovementDeltas.applyTempAt((int)x,(int)y,(int)z, facing);
}



void sn_PushableBlock::restoreOriginalMovementDeltas()
{
	if ( gone || fallingOffStage ) return;
	//restore originals that were replaced when block had moved here
	originalMovementDeltas.applyAt((int)oldX, (int)oldY, (int)oldZ, facing);
	oldX = x; oldY = y; oldZ = z; //update old vertex so it can be restored later
}

void sn_PushableBlock::applyMovementDeltas()
{
	if ( gone || fallingOffStage ) return;
	//apply the block into the movement array writing over the previous values
	blockMovementDeltas.applyTempAt((int)x,(int)y,(int)z, facing);
}

void sn_PushableBlock::recordOriginalMovementDeltas()
{
	if ( gone || fallingOffStage ) return;
	//record the original deltas that are under where the block is being moved to
	originalMovementDeltas.clear();
	for (vector<sn_movementDelta>::const_iterator ditr = blockMovementDeltas.getDeltasBeginItr();
		 ditr != blockMovementDeltas.getDeltasEndItr(); ++ditr)
	{
		sn_movementDelta tempDelta;
		tempDelta = (*ditr);
		tempDelta.newMoveType = pMovementArray->getMovement((int)x+tempDelta.xOffset, (int)y+tempDelta.yOffset,
															(int)z+tempDelta.zOffset, tempDelta.movementDirection);
		originalMovementDeltas.add(tempDelta);
	}
}	