#include "sn_Screen.h"

//initialize SDL and OpenGL and change video to requested settings                                use 0 for colorDepth to use current color depth
int sn_Screen::init(Uint16 gameWidth, Uint16 gameHeight, Uint16 screenWidth, Uint16 screenHeight, Uint16 colorDepth, int useWindow, const char * windowName)
{
	/* init SDL
	SDL is useful for cross platform video/audio/joystick/network code.
	Should work with linked libraries on Mac, Windows, Linux, etc.
	combined with OpenGL, we can have a completely portable app 
	*/
	 
	SDL_Surface * screen; //Pointer to the screen buffer
	
	resW = screenWidth;
	resH = screenHeight;
	gameW = gameWidth;
	gameH = gameHeight;
	depth = colorDepth;
	window = useWindow;
	strcpy(name, windowName);
	
	//flags for initializing using SDL_SetVideoMode
	Uint32 initFlags = SDL_INIT_VIDEO;
	initFlags |= SDL_INIT_AUDIO;
	//initFlags |= SDL_INIT_JOYSTICK;
	Uint32 videoFlags = SDL_OPENGL;    // enable OpenGL in SDL
	//videoFlags |= SDL_HWPALETTE;     // store palette in hardware
	
	if (!useWindow)
	{
		videoFlags |= SDL_FULLSCREEN;      // start in fullscreen
	}
	//videoFlags |= SDL_RESIZABLE;       // enable window resizing
	//videoFlags |= SDL_GL_DOUBLEBUFFER; // enable double buffering  **does nothing in OS X currently
	
	//* initialize SDL
	if ( SDL_Init( initFlags ) < 0 )
	{
		fprintf( stderr, "SDL_Init() failed: %s\n", SDL_GetError() );
		Quit(1);
	}
	
	//set window name, (and icon if not null)
	SDL_WM_SetCaption(name, NULL);
	
	//some OpenGL attributes need to be set BEFORE SDL's video mode is set
	// Sets up OpenGL double buffering *currently does nothing on Mac OS X
	//SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );
	SDL_GL_SetAttribute( SDL_GL_RED_SIZE, 8 );
	SDL_GL_SetAttribute( SDL_GL_GREEN_SIZE, 8 );
	SDL_GL_SetAttribute( SDL_GL_BLUE_SIZE, 8 );
	SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE, 16 );
	// set bpp to whatever format screen is already in
	//video_bpp = videoInfo->vfmt->BitsPerPixel;
	
	screen = SDL_SetVideoMode(screenWidth, screenHeight, colorDepth, videoFlags);
	
	// Make sure there is a screen
	if (!screen)
	{
		fprintf(stderr,  "SDL_SetVideoMode() failed: %s\n", SDL_GetError());
		Quit(1);
	}
	
	//sync to vbl
	#if defined(__APPLE__)
	const long VBL = 1;
	CGLSetParameter(CGLGetCurrentContext(), kCGLCPSwapInterval, &VBL);
	#endif
	/*		
	Using OpenGL for 2D video applications is more complicated initially but the
	advantages are extreme:
	-Hardware acceleration available on all platforms
	-Scaling, Rotation, Transparency, Antialiasing in some cases for almost no performance cost.
	-capability to mix 2D with 3D imagery easily
	-Huge amount of online documentation, tutorials, and support available for free
	*/
	// gl init
	glClearDepth(1.0f);                                                     // Depth Buffer Setup
	glDepthFunc(GL_LEQUAL);                                                 // Type Of Depth Testing
	glEnable(GL_DEPTH_TEST);
	glDisable(GL_LIGHTING);
	glShadeModel(GL_SMOOTH);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glEnable(GL_ALPHA_TEST);
	//glAlphaFunc(GL_GREATER, 0.1f);
	glAlphaFunc(GL_GREATER, 1/255.0);
	
	// texture init
	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
	//glEnable(GL_TEXTURE_2D);
	//glMatrixMode(GL_TEXTURE);
	//glLoadIdentity();
//	glScalef(0.125/phf, 0.125/phf, 1.0);	// glAARG requires rectangle coordinate setup for textures (phf is defined in AAPrimitives.h)
	
	//use glAArg for Anti-aliased lines and points
//	glAAInit();
	
//	glAAGenerateAATex(0.0f, 0, false); //crisp AA due to 0.0f falloff setting
//	glAAGenerateAATex(0.5f, 1, false); //softer lines due to 0.5f falloff setting
//	glAAGenerateAATex(1.0f, 2, false); //even softer
									   //now, before we draw lines or points, we can "glBindTexture(GL_TEXTURE_2D, glAA_texture[ A ]);" 
									   //where 'A' is 0,1, or 2 and it will use the different style of softness for the line
	
    // ***GL_VERTEX_ARRAY_RANGE_APPLE undeclared???
	//glAAEnable(GL_VERTEX_ARRAY_RANGE_APPLE);	// we want VAR acceleration
	
	glClearColor(0.2f, 0.12f, 0.0f, 1.0f); // Background color
	
    // Setup our viewport
    glViewport(0, 0, (GLint)screenWidth, (GLint)screenHeight);
	
    // change to the projection matrix and set viewing volume
	glMatrixMode(GL_PROJECTION);        // Select The Projection Matrix
	glLoadIdentity();			// Reset The Projection Matrix
	
	//ortho:
	//gluOrtho2D(0, gameWidth, gameHeight, 0); // Set Up An Orthographic projection (no perspective) using screen coord
	//glPushMatrix();
	//glLoadIdentity();
	//perspective:
	// Calculate The Aspect Ratio Of The Window
	// The parameters are:
	// (view angle, aspect ration of the width to the height, 
	//  The closest distance to the camera before it clips, 
	// FOV        // Ratio                //  The farthest distance before it stops drawing)
    gluPerspective(10.0f,(GLfloat)gameW/(GLfloat)gameH, 20.0f ,2000.0f);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	gluLookAt(0, 0, 6,   0, 0, 0,   0, 1, 0);
	return true;
}

int sn_Screen::resize(Uint16 screenWidth, Uint16 screenHeight, Uint16 colorDepth, int useWindow)
{
	resWOld = resW;
	resHOld = resH;
	windowOld = window;
	depthOld = depth;
	
	return init(gameW, gameH, screenWidth, screenHeight, colorDepth, useWindow, name);
}

void sn_Screen::revert()
{
	resW = resWOld;
	resH = resHOld;
	window = windowOld;
	depth = depthOld;
	
	init(gameW, gameH, resWOld, resHOld, depthOld, windowOld, name); 
}

void sn_Screen::Quit( int returnCode )
{
    // Application Is Finished
    
    // clean up the window
    SDL_Quit( );
	
    // and exit appropriately
    exit( returnCode );
}

Uint16 sn_Screen::getGameW()
{
	return gameW;
}
Uint16 sn_Screen::getGameH()
{
	return gameH;
}
Uint16 sn_Screen::getW()
{
	return resW;
}
Uint16 sn_Screen::getH()
{
	return resH;
}
int sn_Screen::usingWindow()
{
	return window;
}
Uint16 sn_Screen::getColorDepth()
{
	return depth;
}
